@@ -7,13 +7,21 @@
 
 #include "fmt/ranges.h"
 
+#include <array>
+#include <list>
 #include <map>
+#include <numeric>
 #include <queue>
 #include <stack>
 #include <string>
 #include <utility>
 #include <vector>
 
+#if FMT_HAS_INCLUDE(<ranges>)
+#  include <ranges>
+#endif
+
+#include "fmt/format.h"
 #include "gtest/gtest.h"
 
 #if !FMT_GCC_VERSION || FMT_GCC_VERSION >= 601
@@ -51,8 +59,13 @@ TEST(ranges_test, format_vector) {
   EXPECT_EQ(fmt::format("{:n:#x}", v), "0x1, 0x2, 0x3, 0x5, 0x7, 0xb");
 
   auto vc = std::vector<char>{'a', 'b', 'c'};
+  auto vec = std::vector<char>{'a', '\n', '\t'};
   auto vvc = std::vector<std::vector<char>>{vc, vc};
   EXPECT_EQ(fmt::format("{}", vc), "['a', 'b', 'c']");
+  EXPECT_EQ(fmt::format("{:s}", vc), "\"abc\"");
+  EXPECT_EQ(fmt::format("{:?s}", vec), "\"a\\n\\t\"");
+  EXPECT_EQ(fmt::format("{:s}", vec), "\"a\n\t\"");
+  EXPECT_EQ(fmt::format("{::s}", vvc), "[\"abc\", \"abc\"]");
   EXPECT_EQ(fmt::format("{}", vvc), "[['a', 'b', 'c'], ['a', 'b', 'c']]");
   EXPECT_EQ(fmt::format("{:n}", vvc), "['a', 'b', 'c'], ['a', 'b', 'c']");
   EXPECT_EQ(fmt::format("{:n:n}", vvc), "'a', 'b', 'c', 'a', 'b', 'c'");
@@ -77,6 +90,35 @@ TEST(ranges_test, format_map) {
   EXPECT_EQ(fmt::format("{:n}", m), "\"one\": 1, \"two\": 2");
 }
 
+struct test_map_value {};
+
+FMT_BEGIN_NAMESPACE
+template <> struct formatter<test_map_value> : formatter<string_view> {
+  auto format(test_map_value, format_context& ctx) const
+      -> format_context::iterator {
+    return formatter<string_view>::format("foo", ctx);
+  }
+};
+
+template <typename K>
+struct formatter<std::pair<K, test_map_value>> : formatter<string_view> {
+  auto format(std::pair<K, test_map_value>, format_context& ctx) const
+      -> format_context::iterator {
+    return ctx.out();
+  }
+};
+
+template <typename K>
+struct is_tuple_formattable<std::pair<K, test_map_value>, char>
+    : std::false_type {};
+
+FMT_END_NAMESPACE
+
+TEST(ranges_test, format_map_custom_pair) {
+  EXPECT_EQ(fmt::format("{}", std::map<int, test_map_value>{{42, {}}}),
+            "{42: \"foo\"}");
+}
+
 TEST(ranges_test, format_set) {
   EXPECT_EQ(fmt::format("{}", std::set<std::string>{"one", "two"}),
             "{\"one\", \"two\"}");
@@ -242,7 +284,7 @@ template <typename T> class non_const_only_range {
   explicit non_const_only_range(Args&&... args)
       : vec(std::forward<Args>(args)...) {}
 
-  auto begin() -> const_iterator{ return vec.begin(); }
+  auto begin() -> const_iterator { return vec.begin(); }
   auto end() -> const_iterator { return vec.end(); }
 };
 
@@ -281,7 +323,7 @@ TEST(ranges_test, range) {
   EXPECT_EQ(fmt::format("{}", z), "[0, 0, 0]");
 }
 
-enum test_enum { foo };
+enum test_enum { foo, bar };
 auto format_as(test_enum e) -> int { return e; }
 
 TEST(ranges_test, enum_range) {
@@ -296,6 +338,40 @@ TEST(ranges_test, unformattable_range) {
 }
 #endif
 
+TEST(ranges_test, join) {
+  using fmt::join;
+  int v1[3] = {1, 2, 3};
+  auto v2 = std::vector<float>();
+  v2.push_back(1.2f);
+  v2.push_back(3.4f);
+  void* v3[2] = {&v1[0], &v1[1]};
+
+  EXPECT_EQ(fmt::format("({})", join(v1, v1 + 3, ", ")), "(1, 2, 3)");
+  EXPECT_EQ(fmt::format("({})", join(v1, v1 + 1, ", ")), "(1)");
+  EXPECT_EQ(fmt::format("({})", join(v1, v1, ", ")), "()");
+  EXPECT_EQ(fmt::format("({:03})", join(v1, v1 + 3, ", ")), "(001, 002, 003)");
+  EXPECT_EQ("(+01.20, +03.40)",
+            fmt::format("({:+06.2f})", join(v2.begin(), v2.end(), ", ")));
+
+  EXPECT_EQ(fmt::format("{0:{1}}", join(v1, v1 + 3, ", "), 1), "1, 2, 3");
+
+  EXPECT_EQ(fmt::format("{}, {}", v3[0], v3[1]),
+            fmt::format("{}", join(v3, v3 + 2, ", ")));
+
+  EXPECT_EQ(fmt::format("({})", join(v1, ", ")), "(1, 2, 3)");
+  EXPECT_EQ(fmt::format("({:+06.2f})", join(v2, ", ")), "(+01.20, +03.40)");
+
+  auto v4 = std::vector<test_enum>{foo, bar, foo};
+  EXPECT_EQ(fmt::format("{}", join(v4, " ")), "0 1 0");
+}
+
+#ifdef __cpp_lib_byte
+TEST(ranges_test, join_bytes) {
+  auto v = std::vector<std::byte>{std::byte(1), std::byte(2), std::byte(3)};
+  EXPECT_EQ(fmt::format("{}", fmt::join(v, ", ")), "1, 2, 3");
+}
+#endif
+
 #ifdef FMT_RANGES_TEST_ENABLE_JOIN
 TEST(ranges_test, join_tuple) {
   // Value tuple args.
@@ -360,7 +436,7 @@ struct cpp20_only_range {
     iterator() = default;
     iterator(int i) : val(i) {}
     auto operator*() const -> int { return val; }
-    auto operator++() -> iterator&{
+    auto operator++() -> iterator& {
       ++val;
       return *this;
     }
@@ -414,8 +490,33 @@ TEST(ranges_test, join_range) {
       "0,1,2,3,4");
 #  endif
 }
+
+namespace adl {
+struct vec {
+  int n[2] = {42, 43};
+};
+
+auto begin(const vec& v) -> const int* { return v.n; }
+auto end(const vec& v) -> const int* { return v.n + 2; }
+}  // namespace adl
+
+TEST(ranges_test, format_join_adl_begin_end) {
+  EXPECT_EQ(fmt::format("{}", fmt::join(adl::vec(), "/")), "42/43");
+}
+
 #endif  // FMT_RANGES_TEST_ENABLE_JOIN
 
+#if defined(__cpp_lib_ranges) && __cpp_lib_ranges >= 202207L
+TEST(ranges_test, nested_ranges) {
+  auto l = std::list{1, 2, 3};
+  auto r = std::views::iota(0, 3) | std::views::transform([&l](auto i) {
+             return std::views::take(std::ranges::subrange(l), i);
+           }) |
+           std::views::transform(std::views::reverse);
+  EXPECT_EQ(fmt::format("{}", r), "[[], [1], [2, 1]]");
+}
+#endif
+
 TEST(ranges_test, is_printable) {
   using fmt::detail::is_printable;
   EXPECT_TRUE(is_printable(0x0323));
@@ -430,7 +531,7 @@ TEST(ranges_test, escape) {
   EXPECT_EQ(fmt::format("{}", vec{"\x7f"}), "[\"\\x7f\"]");
   EXPECT_EQ(fmt::format("{}", vec{"n\xcc\x83"}), "[\"n\xcc\x83\"]");
 
-  if (fmt::detail::is_utf8()) {
+  if (fmt::detail::use_utf8()) {
     EXPECT_EQ(fmt::format("{}", vec{"\xcd\xb8"}), "[\"\\u0378\"]");
     // Unassigned Unicode code points.
     EXPECT_EQ(fmt::format("{}", vec{"\xf0\xaa\x9b\x9e"}), "[\"\\U0002a6de\"]");
@@ -528,3 +629,122 @@ TEST(ranges_test, container_adaptor) {
     EXPECT_EQ(fmt::format("{}", m), "[1, 2]");
   }
 }
+
+struct tieable {
+  int a = 3;
+  double b = 0.42;
+};
+
+auto format_as(const tieable& t) -> std::tuple<int, double> {
+  return std::tie(t.a, t.b);
+}
+
+TEST(ranges_test, format_as_tie) {
+  EXPECT_EQ(fmt::format("{}", tieable()), "(3, 0.42)");
+}
+
+struct lvalue_qualified_begin_end {
+  int arr[5] = {1, 2, 3, 4, 5};
+
+  auto begin() & -> const int* { return arr; }
+  auto end() & -> const int* { return arr + 5; }
+};
+
+TEST(ranges_test, lvalue_qualified_begin_end) {
+  EXPECT_EQ(fmt::format("{}", lvalue_qualified_begin_end{}), "[1, 2, 3, 4, 5]");
+}
+
+#if !defined(__cpp_lib_ranges) || __cpp_lib_ranges <= 202106L
+#  define ENABLE_STD_VIEWS_TESTS 0
+#elif FMT_CLANG_VERSION
+#  if FMT_CLANG_VERSION > 1500
+#    define ENABLE_STD_VIEWS_TESTS 1
+#  else
+#    define ENABLE_STD_VIEWS_TESTS 0
+#  endif
+#else
+#  define ENABLE_STD_VIEWS_TESTS 1
+#endif
+
+#if ENABLE_STD_VIEWS_TESTS
+TEST(ranges_test, input_range_join) {
+  auto iss = std::istringstream("1 2 3 4 5");
+  auto view = std::views::istream<std::string>(iss);
+  EXPECT_EQ("1, 2, 3, 4, 5",
+            fmt::format("{}", fmt::join(view.begin(), view.end(), ", ")));
+}
+
+TEST(ranges_test, input_range_join_overload) {
+  auto iss = std::istringstream("1 2 3 4 5");
+  EXPECT_EQ(
+      "1.2.3.4.5",
+      fmt::format("{}", fmt::join(std::views::istream<std::string>(iss), ".")));
+}
+
+namespace views_filter_view_test {
+struct codec_mask {
+  static constexpr auto codecs = std::array{0, 1, 2, 3};
+  int except = 0;
+};
+
+auto format_as(codec_mask mask) {
+  // Careful not to capture param by reference here, it will dangle.
+  return codec_mask::codecs |
+         std::views::filter([mask](auto c) { return c != mask.except; });
+}
+}  // namespace views_filter_view_test
+
+TEST(ranges_test, format_as_with_ranges_mutable_begin_end) {
+  using namespace views_filter_view_test;
+  {
+    auto make_filter_view = []() {
+      return codec_mask::codecs |
+             std::views::filter([](auto c) { return c != 2; });
+    };
+    auto r = make_filter_view();
+    EXPECT_EQ("[0, 1, 3]", fmt::format("{}", r));
+    EXPECT_EQ("[0, 1, 3]", fmt::format("{}", make_filter_view()));
+  }
+
+  {
+    auto mask = codec_mask{2};
+    const auto const_mask = codec_mask{2};
+
+    EXPECT_EQ("[0, 1, 3]", fmt::format("{}", mask));
+    EXPECT_EQ("[0, 1, 3]", fmt::format("{}", const_mask));
+    EXPECT_EQ("[0, 1, 3]", fmt::format("{}", codec_mask{2}));
+  }
+}
+
+#endif
+
+TEST(ranges_test, std_istream_iterator_join) {
+  auto&& iss = std::istringstream("1 2 3 4 5");
+  auto first = std::istream_iterator<int>(iss);
+  auto last = std::istream_iterator<int>();
+  EXPECT_EQ("1, 2, 3, 4, 5", fmt::format("{}", fmt::join(first, last, ", ")));
+}
+
+TEST(ranges_test, movable_only_istream_iter_join) {
+  // Mirrors C++20 std::ranges::basic_istream_view::iterator.
+  struct noncopyable_istream_iterator : std::istream_iterator<int> {
+    explicit noncopyable_istream_iterator(std::istringstream& iss)
+        : std::istream_iterator<int>{iss} {}
+    noncopyable_istream_iterator(const noncopyable_istream_iterator&) = delete;
+    noncopyable_istream_iterator(noncopyable_istream_iterator&&) = default;
+  };
+  static_assert(
+      !std::is_copy_constructible<noncopyable_istream_iterator>::value, "");
+
+  auto&& iss = std::istringstream("1 2 3 4 5");
+  auto first = noncopyable_istream_iterator(iss);
+  auto last = std::istream_iterator<int>();
+  EXPECT_EQ("1, 2, 3, 4, 5",
+            fmt::format("{}", fmt::join(std::move(first), last, ", ")));
+}
+
+struct not_range {
+  void begin() const {}
+  void end() const {}
+};
+static_assert(!fmt::is_formattable<not_range>{}, "");
