@@ -21,9 +21,12 @@
 #include <errno.h>
 
 #if !defined(WIN32) && !defined(_MSC_VER)
-#include <sys/socket.h> // socket before net/if.h for mac
 #include <net/if.h>
 #include <sys/ioctl.h>
+#include <sys/socket.h> // socket before net/if.h for mac
+
+#include <poll.h>
+
 #else
 typedef int socklen_t;
 #include <winsock2.h>
@@ -582,53 +585,27 @@ int artnet_net_start(node n) {
 /*
  * Receive a packet.
  */
-int artnet_net_recv(node n, artnet_packet p, int delay) {
-  ssize_t len;
-  struct sockaddr_in cliAddr;
-  socklen_t cliLen = sizeof(cliAddr);
-  fd_set rset;
-  struct timeval tv;
-  int maxfdp1 = n->sd + 1;
-
-  FD_ZERO(&rset);
-  FD_SET((unsigned int) n->sd, &rset);
-
-  tv.tv_usec = 0;
-  tv.tv_sec = delay;
-  p->length = 0;
-
-  switch (select(maxfdp1, &rset, NULL, NULL, &tv)) {
-    case 0:
-      // timeout
-      return RECV_NO_DATA;
-      break;
-    case -1:
-      if ( errno != EINTR) {
-        artnet_error("Select error %s", artnet_net_last_error());
-        return ARTNET_ENET;
-      }
-      return ARTNET_EOK;
-      break;
-    default:
-      break;
-  }
 
+int artnet_net_recv_read(node n, artnet_packet p)
+{
   // need a check here for the amount of data read
   // should prob allow an extra byte after data, and pass the size as sizeof(Data) +1
   // then check the size read and if equal to size(data)+1 we have an error
-  len = recvfrom(n->sd,
-                 (char*) &(p->data), // char* for win32
-                 sizeof(p->data),
-                 0,
-                 (SA*) &cliAddr,
-                 &cliLen);
-  if (len < 0) {
+  struct sockaddr_in cliAddr;
+  socklen_t cliLen = sizeof(cliAddr);
+  ssize_t len = recvfrom(
+      n->sd,
+      (char*)&(p->data), // char* for win32
+      sizeof(p->data), 0, (SA*)&cliAddr, &cliLen);
+  if(len < 0)
+  {
     artnet_error("Recvfrom error %s", artnet_net_last_error());
     return ARTNET_ENET;
   }
 
-  if (cliAddr.sin_addr.s_addr == n->state.ip_addr.s_addr ||
-      ntohl(cliAddr.sin_addr.s_addr) == LOOPBACK_IP) {
+  if(cliAddr.sin_addr.s_addr == n->state.ip_addr.s_addr
+     || ntohl(cliAddr.sin_addr.s_addr) == LOOPBACK_IP)
+  {
     p->length = 0;
     return ARTNET_EOK;
   }
@@ -639,6 +616,71 @@ int artnet_net_recv(node n, artnet_packet p, int delay) {
   return ARTNET_EOK;
 }
 
+int artnet_net_recv(node n, artnet_packet p, int delay)
+{
+  p->length = 0;
+
+#if !defined(_WIN32)
+  {
+    struct pollfd pollfds[1] = {{.fd = n->sd, .events = POLLIN, .revents = 0}};
+
+    if(poll(pollfds, 1, delay * 1000) < 0)
+    {
+      if(errno != EINTR)
+      {
+        artnet_error("poll error %s", artnet_net_last_error());
+        return ARTNET_ENET;
+      }
+    }
+    if(pollfds[0].revents & POLLERR)
+    {
+      if(errno != EINTR)
+      {
+        artnet_error("poll error %s", artnet_net_last_error());
+        return ARTNET_ENET;
+      }
+      return ARTNET_EOK;
+    }
+    if(!(pollfds[0].revents & POLLIN))
+    {
+      return RECV_NO_DATA;
+    }
+  }
+#else
+  // FIXME WSAPoll...
+  {
+    struct timeval tv;
+    tv.tv_usec = 0;
+    tv.tv_sec = delay;
+
+    fd_set rset;
+    int maxfdp1 = n->sd + 1;
+
+    FD_ZERO(&rset);
+    FD_SET((unsigned int)n->sd, &rset);
+
+    switch(select(maxfdp1, &rset, NULL, NULL, &tv))
+    {
+      case 0:
+        // timeout
+        return RECV_NO_DATA;
+        break;
+      case -1:
+        if(errno != EINTR)
+        {
+          artnet_error("Select error %s", artnet_net_last_error());
+          return ARTNET_ENET;
+        }
+        return ARTNET_EOK;
+        break;
+      default:
+        break;
+    }
+  }
+#endif
+
+  return artnet_net_recv_read(n, p);
+}
 
 /*
  * Send a packet.
