@@ -18,15 +18,16 @@ docker run --rm -it \
            ossia/score-linux-llvm \
            /bin/bash Recipe
 
-wget "https://github.com/probonopd/AppImageKit/releases/download/continuous/appimagetool-x86_64.AppImage" -O appimagetool-x86_64.AppImage 
-chmod a+x appimagetool-x86_64.AppImage
-wget "https://github.com/probonopd/AppImageKit/releases/download/continuous/AppRun-x86_64" -O AppRun-x86_64
-chmod a+x AppRun-x86_64
-sudo chown -R $(whoami) /tmp/build
-cp AppRun-x86_64 /tmp/build/score.AppDir/AppRun
-cp ossia-score.desktop /tmp/build/score.AppDir/
-cp $SOURCE/src/lib/resources/ossia-score.png /tmp/build/score.AppDir/
-cp $SOURCE/src/lib/resources/ossia-score.png /tmp/build/score.AppDir/.DirIcon
-cp -rf /tmp/build/score.AppDir .
-./appimagetool-x86_64.AppImage -n score.AppDir Score.AppImage
 
+wget "https://github.com/AppImage/appimagetool/releases/download/continuous/appimagetool-x86_64.AppImage" -O appimagetool-x86_64.AppImage 
+chmod a+x appimagetool-x86_64.AppImage
+wget "https://github.com/AppImage/type2-runtime/releases/download/continuous/runtime-x86_64" -O runtime-x86_64
+chmod a+x runtime-x86_64
+sudo chown -R "$(whoami)" /tmp/build
+cp runtime-x86_64 /tmp/build/score.AppDir/runtime-x86_64
+cp ossia-score.desktop /tmp/build/score.AppDir/
+cp "$SOURCE/src/lib/resources/ossia-score.png" /tmp/build/score.AppDir/
+cp "$SOURCE/src/lib/resources/ossia-score.png" /tmp/build/score.AppDir/.DirIcon
+cp -rf /tmp/build/score.AppDir .
+
+./appimagetool-x86_64.AppImage -n "score.AppDir" "Score.AppImage" --runtime-file  runtime-x86_64
