@@ -16,4 +16,4 @@ PROCESS_METADATA(
     "Automation following a 3D curve", "ossia score",
     (QStringList{"Curve", "Automation", "3D"}), {},
     {std::vector<Process::PortType>{Process::PortType::Message}},
-    Process::ProcessFlags::SupportsTemporal)
+    Process::ProcessFlags::SupportsLasting)
